""" Copyright 2012, 2013 UW Information Technology, University of Washington

    Licensed under the Apache License, Version 2.0 (the "License");
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

    Unless required by applicable law or agreed to in writing, software
    distributed under the License is distributed on an "AS IS" BASIS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.
"""

from spotseeker_server.test.images import ImageTestCase
from django.core.files import File
from django.conf import settings
from django.test.client import Client
from spotseeker_server.models import Spot, SpotImage
from io import BytesIO
from PIL import Image
import os
from django.test.utils import override_settings


TEST_ROOT = os.path.abspath(os.path.dirname(__file__))


@override_settings(SPOTSEEKER_AUTH_MODULE='spotseeker_server.auth.all_ok',
                   SPOTSEEKER_AUTH_ADMINS=('demo_user',),
                   MEDIA_ROOT=os.path.join(TEST_ROOT, "..", "resources"))
class ImageThumbTest(ImageTestCase):

    def setUp(self):
        spot = Spot.objects.create(
            name="This is to test thumbnailing images"
        )
        self.spot = spot
        self.url = '/api/v1/spot/%s/image' % spot.pk

    def assertContentType(self, response, expected_type):

        self.assertEqual(
            response['Content-type'],
            expected_type,
            'Got wrong content type for response'
        )

    def test_all_thumbs(self):
        c = self.client

        image_formats = (
            ('image/jpeg', 'JPEG', '%s/test_jpeg.jpg' % settings.MEDIA_ROOT),
            ('image/png', 'PNG', '%s/test_png.png' % settings.MEDIA_ROOT),
            ('image/gif', 'GIF', '%s/test_gif.gif' % settings.MEDIA_ROOT),
        )
        # Width/height pairs of dimensions which should return an image with
        # the original width/height
        good_dimensions = (
            (100, 100),
            (1, 100),
            (100, 1),
            (1, 1),
            (200, 200)
        )

        # Bad pairs which should result in 400s
        # Width, Height, Comment
        bad_dimensions = (
            (100, 0, 'missing height'),
            (0, 100, 'missing width'),
            (0, 0, 'missing both'),
            (100, -100, 'negative height'),
            (-100, 100, 'negative width'),
            ('a', 100, 'invalid width'),
            (100, 'a', 'invalid height')
        )

        for content_type, img_format, image_name in image_formats:
            response = self.upload_image(
                image_name, self.url,
                {'description': 'this is a %s' % img_format})
            print(response.content)
            new_base_location = response["Location"]

            for width, height in good_dimensions:
                img_url = '%s/thumb/%sx%s' % (new_base_location, width, height)
                response = c.get(img_url)

                self.assertEqual(
                    response.status_code,
                    200,
                    'Expected 200 for %sx%s (%s)'
                    % (width, height, img_format))
                self.assertContentType(response, content_type)
                data = BytesIO(response.content)
                im = Image.open(data)
                self.assertEqual(im.format, img_format,
                                 'img format did not match')
                self.assertEqual(im.size[0], width,
                                 'img width did not match (%s)'
                                  % img_format)
                self.assertEqual(im.size[1], height,
                                 'img height did not match (%s)'
                                 % img_format)

            for width, height, comment in bad_dimensions:

                img_url = '%s/thumb/%sx%s' % (new_base_location, width, height)
                response = c.get(img_url)
                self.assertEqual(response.status_code,
                                 400,
                                 'Expected 400 for image with width %s and'
                                 ' height %s (%s)' % (width, height, comment))

    def test_invalid_url(self):
        c = Client()
        bad_spot = Spot.objects.create(name="This is the wrong spot")

        spot = Spot.objects.create(name="This is to test getting images")

        with open('%s/../resources/test_gif.gif' % TEST_ROOT, 'rb') as f:
            gif = SpotImage.objects.create(
                description='This is the GIF test',
                spot=spot,
                image=File(f))

        url = '/api/v1/spot/%s/image/%s/thumb/10x10' % (bad_spot.pk, gif.pk)
        response = c.get(url)
        self.assertEqual(
            response.status_code,
            404,
            "Give a 404 for a spot id that doesn't match the image's"
        )

    def assertImageRatio(self, im, ratio):
        im_ratio = float(im.size[0]) / float(im.size[1])
        self.assertAlmostEqual(im_ratio, ratio, places=1)

    def test_constrain_jpeg(self):
        self._test_aspect_ratio('image/jpeg',
                                'JPEG',
                                '%s/test_jpeg2.jpg' % settings.MEDIA_ROOT)

    def test_constrain_png(self):
        self._test_aspect_ratio('image/png',
                                'PNG',
                                '%s/test_png2.png' % settings.MEDIA_ROOT)

    def test_constrain_gif(self):
        self._test_aspect_ratio('image/gif',
                                'GIF',
                                '%s/test_gif2.gif' % settings.MEDIA_ROOT)

    def _test_aspect_ratio(self, content_type, img_format, img_file):
        c = self.client
        response = self.upload_image(
            img_file,
            self.url,
            {'description': 'test %s #2' % img_format})

        orig_im = Image.open(img_file)
        orig_ratio = float(orig_im.size[0]) / float(orig_im.size[1])

        new_base_location = response['Location']

        def get_thumb_constrained(**kwargs):
            args = ','.join('%s:%s' % x for x in list(kwargs.items()))
            url = '%s/thumb/constrain/%s' % (new_base_location, args)
            response = c.get(url)
            return response

        # 50 width test
        response = get_thumb_constrained(width=50)
        self.assertContentType(response, content_type)
        im = Image.open(BytesIO(response.content))
        self.assertEqual(im.size[0], 50, '%s Should have been 50 width')
        self.assertImageRatio(im, orig_ratio)
        self.assertEqual(im.format, img_format, '%s Format was not the same')

        # 50 height test
        response = get_thumb_constrained(height=50)
        self.assertContentType(response, content_type)
        im = Image.open(BytesIO(response.content))
        self.assertEqual(im.size[1], 50, 'Should have been 50 height')
        self.assertImageRatio(im, orig_ratio)
        self.assertEqual(im.format, img_format, 'Format was not the same')

        # 75 width, 50 height
        # Should result in 50 height
        response = get_thumb_constrained(width=75, height=50)
        self.assertContentType(response, content_type)
        im = Image.open(BytesIO(response.content))
        self.assertEqual(im.size[1], 50, 'Should have been 50 height')
        self.assertImageRatio(im, orig_ratio)
        self.assertEqual(im.format, img_format, 'Format was not the same')

        # 50 width, 75 height
        # Should result in 50 height
        response = get_thumb_constrained(width=50, height=75)
        self.assertContentType(response, content_type)
        im = Image.open(BytesIO(response.content))
        self.assertEqual(im.size[1], 75, 'Should have been 75 height')
        self.assertImageRatio(im, orig_ratio)
        self.assertEqual(im.format, img_format, 'Format was not the same')
